% misc.spc - interface specifications for `misc' abstractions
%
% This file was derived from PCLU distribution (as of 11/18/93), and
% has been modified by Department of Mathematical and Computing
% Sciences, Tokyo Institute of Technology.
%
% Conditions for using, copying, modifying, and distributing this
% file are the same as those of PCLU.
%
% $Id: misc.spc,v 2.0 1996/06/10 09:02:35 ushijima Exp $


%  Copyright	Massachusetts Institute of Technology     1989
% Copyright Massachusetts Institute of Technology 1980, 1989


% The following table shows the file in PCLU distribution from which the
% interface specification for each abstraction was derived.
%
%	==================================================
%	Abstraction name	File name in PCLU
%	--------------------------------------------------
%	_case			lib/clu/scase.clu
%	_host_system		lib/clu/os.clu
%	_int_range		lib/clu/_int_range.clu
%	_operating_system	lib/clu/os.clu
%	ago			lib/clu/dparse.clu
%	bigint			lib/clu/bigint.clu
%	capitalize		lib/clu/scase.clu
%	date_parse		lib/clu/dparse.clu
%	date_sub		lib/clu/dparse.clu
%	heap			lib/heap.spc
%	lex_scan		lib/clu/lex_scan.clu
%	lower			lib/clu/string_ops.clu
%	lower_case		lib/clu/scase.clu
%	padl			lib/clu/string_ops.clu
%	padlc			lib/clu/string_ops.clu
%	padr			lib/clu/string_ops.clu
%	padrc			lib/clu/string_ops.clu
%	parse_line		lib/clu/parse_line.clu
%	quick_sort		lib/clu/quick_sort.clu
%	quicksort		lib/clu/quicksort.clu
%	random			lib/random.spc
%	remblank		lib/clu/string_ops.clu
%	slower			lib/clu/string_ops.clu
%	split_line		lib/clu/split_line.clu
%	string_lindexs		lib/clu/string_lindexs.clu
%	supper			lib/clu/string_ops.clu
%	trim_both		lib/clu/strim.clu
%	trim_head		lib/clu/strim.clu
%	trim_tail		lib/clu/strim.clu
%	upper			lib/clu/string_ops.clu
%	upper_case		lib/clu/scase.clu
%	verify			lib/clu/string_ops.clu
%	xref			lib/clu/xref.clu
%	--------------------------------------------------


% Data abstractions


% Bigint

bigint = cluster is add, sub, mul, minus, abs, max, min, div, mod, power, lt,
	 le, ge, gt, equal, similar, copy, encode, decode, i2bi, bi2i, parse,
	 unparse

% Operations

add = proc(X, Y: bigint) returns(bigint)
	end add

sub = proc(X, Y: bigint) returns(bigint)
	end sub

mul = proc(X, Y: bigint) returns(bigint)
	end mul

minus = proc(X: bigint) returns(bigint)
	end minus

abs = proc(X: bigint) returns(bigint)
	end abs

max = proc(X, Y: bigint) returns(bigint)
	end max

min = proc(X, Y: bigint) returns(bigint)
	end min

div = proc(X, Y: bigint) returns(bigint) signals(zero_divide)
	end div

mod = proc(X, Y: bigint) returns(bigint) signals(zero_divide)
	end mod

power = proc(X, Y: bigint) returns(bigint) signals(negative_exponent)
	end power

lt = proc(X, Y: bigint) returns(bool)
	end lt

le = proc(X, Y: bigint) returns(bool)
	end le

ge = proc(X, Y: bigint) returns(bool)
	end ge

gt = proc(X, Y: bigint) returns(bool)
	end gt

equal = proc(X, Y: bigint) returns(bool)
	end equal

similar = proc(X, Y: bigint) returns(bool)
	end similar

copy = proc(X: bigint) returns(bigint)
	end copy

encode = proc(X: bigint, IST: istream) signals(not_possible(string))
	end encode

decode = proc(IST: istream) returns(bigint)
	 signals(end_of_file, not_possible(string))
	end decode

i2bi = proc(I: int) returns(bigint)
	end i2bi

bi2i = proc(X: bigint) returns(int) signals(overflow)
	end bi2i

parse = proc(S: string) returns(bigint) signals(bad_format)
	end parse

unparse = proc(X: bigint) returns(string)
	end unparse

end bigint


% Heap

heap = cluster[T: type] is create, top, size, empty, insert, remove, copy

% Operations

create = proc(P: proctype(T, T) returns(bool)) returns(heap[T])
	% Create a heap with a particular sorting predicate
	end create
    
top = proc(X: heap[T]) returns(T) signals(empty)
	% Return the best element
	end top
    
size = proc(X: heap[T]) returns(int)
	% Return the number of elements
	end size
    
empty = proc(X: heap[T]) returns(bool)
	% Return true if there are no elements
	end empty
    
insert = proc(X: heap[T], V: T)
	% Insert an element of type T
	end insert
    
remove = proc(X: heap[T]) returns(T) signals(empty)
	% Remove the best element and return it
	end remove
    
copy = proc(X: heap[T]) returns(heap[T])
	end copy
    
end heap


% Random

random = cluster is reset, seed, next

% Operations

reset = proc()
	% restore initial seed
	end reset

seed = proc(X: int)
	% modify current seed
	end seed

next = proc(X: int) returns(int)
	% 0 <= res < x
	end next

end random


% Xref

xref = cluster is create, add_ref, add_refs, rem_refs, elements, keys, output,
       merge, flush

% Operations

create = proc() returns(xref)
	% Create a cross-reference table
	end create

add_ref = proc(TAB: xref, KEY, USER: string)
	% Add a (key, user) pair.  Duplications are ignored.
	end add_ref

add_refs = proc(TAB: xref, KEY: string, USERS: array[string])
	% Add a bunch of (key, user) pairs.
	end add_refs

rem_refs = proc(TAB: xref, KEY: string) returns(array[string])
	% Remove and return all users of the given key
	end rem_refs

elements = iter(TAB: xref) yields(string, string)
	% Yield all (key, user) pairs.
	end elements

keys = iter(TAB: xref) yields(string)
	% Yield all keys
	end keys

output = proc(TAB: xref, LHEAD, RHEAD: string, ST: stream)
	 signals(not_possible(string))
	% Output all references.
	end output

merge = proc(TAB1, TAB2: xref)
	% Merge the second table into the first.  The second table is not
	% altered.
	end merge

flush = proc(TAB: xref)
	% Flush all references
	end flush

end xref


% Procedural/control abstractions


_case = proc(S: string, FIRST, REST: bool) returns(string)
	end _case

_host_system = proc() returns(string)
	end _host_system

_int_range = proc() returns(int, int)
	end _int_range

_operating_system = proc() returns(string)
	end _operating_system

ago = proc(DAYS, MONTHS, YEARS: int) returns(string)
	end ago

capitalize = proc(S: string) returns(string)
	end capitalize

date_parse = proc(S: string) returns(date) signals(invalid_format)
	end date_parse

date_sub = proc(D: date, DAYS, MONTHS, YEARS: int) returns(date)
	   signals(year_too_small, overflow)
	end date_sub

lex_scan = proc(S: string, SEP_CHARS,
		FILL_CHARS, BEGINQ, ENDQ: string,
		COMMENT_CHAR: char)
	   returns(array[string])
	   signals(unterminated_quote(array[string], char))
	end lex_scan

lower = proc(C: char) returns(char)
	end lower

lower_case = proc(S: string) returns(string)
	end lower_case

padl = proc(S: string, W: int) returns(string)
	end padl

padlc = proc(S: string, W: int, C: char) returns(string)
	end padlc

padr = proc(S: string, W: int) returns(string)
	end padr

padrc = proc(S: string, W: int, C: char) returns(string)
	end padrc

parse_line = proc(S: string) returns(array[string])
	end parse_line

quick_sort = proc[T: type](ITEMS: array[T],
			   LOW, HIGH: int,
			   LESS, EQUAL: proctype(t, t) returns(bool))
	end quick_sort

quicksort = proc[T: type](ITEMS: array[T],
			  LOW, HIGH: int,
			  LESS: proctype(T, T) returns(bool))
	    signals(bounds)
	end quicksort

remblank = proc(S: string) returns(string)
	end remblank

sisort = proc[T: type](ITEMS: array[T],
		       LOW, HIGH: int,
		       LESS: proctype(T, T) returns(bool))
	 signals (bounds)
	end sisort

slower = proc(S: string) returns(string)
	end slower

split_line = proc(LINE: string, SEP, QUOTE: char) returns(array[string])
	end split_line

string_lindexs = proc(PAT: string, S: string) returns(int)
	end string_lindexs

supper = proc(S: string) returns(string)
	end supper

trim_both = proc(S: string) returns(string)
	end trim_both

trim_head = proc(S: string) returns(string)
	end trim_head

trim_tail = proc(S: string) returns(string)
	end trim_tail

upper = proc(C: char) returns(char)
	end upper

upper_case = proc(S: string) returns(string)
	end upper_case

verify = proc(S: string, CHARS: string) returns(bool)
	end verify
